@@ -18,12 +18,16 @@
 
 #undef HAVE_GETHOSTNAME
 
-#undef HAVE_LIBNCURSES
+#undef HAVE_LIBCURSES
 #undef HAVE_LIBX11
 
+#undef HAVE_NCURSES_H
+
+#undef MAPPATH		/* Systemwide path for maps.  */
+
 #undef VERSION_STR	/* Version string in the form "motti x.x".
 			   Best defined at compile-time since it's
-			   fixed for one executable.  */
+			   fixed for the executable.  */
 
 ^L
 /* Leave that blank line there!!  Autoheader needs it.
