@@ -18,10 +18,6 @@
 
 #include <config.h>
 
-#include "curses_if.h"
-#include "map.h"
-#include "occupy.h"
-
 #ifdef HAVE_NCURSES_H
 #include <ncurses.h>
 #else
@@ -29,33 +25,39 @@
 #endif
 
 #include <stdlib.h>
+#include <unistd.h>
+#ifndef _POSIX_JOB_CONTROL
+#include <stdio.h>
+#endif
+
+#include "curses_if.h"
+#include "map.h"
+#include "occupy.h"
+#include "comm.h"
 
 enum colors {
   COL_SEA = 1,	/* Sea.  */
   COL_ME,	/* Player in turn.  */
+  COL_REMOTE,	/* Remote active player.  */
   COL_OTHER,	/* Other players.  */
   /* TODO: change invade routines to support these two.  */
   COL_ENC,	/* Last encircled and occupied cells.  */
   COL_O_ENC,	/* Last encircled cells in other players' control.  */
   COL_CAP,	/* Color of capital.  */
-  COL_O_CAP,	/* Color of other players' capitals.  */
+  COL_R_CAP,	/* Same, remote active player.  */
+  COL_O_CAP,	/* Same, inactive players.  */
   COL_OCC,	/* Color of occupied areas.  */
-  COL_O_OCC	/* Color of other players' occupied areas.  */
+  COL_R_OCC,	/* Same, remote active player.  */
+  COL_O_OCC	/* Same, inactive players.  */
 };
 
-enum refresh_amount {NONE, CURSOR, STATUSLINE, REFRESH_MAP, ALL};
-
-static void kill_curses (void);
 static chtype choose_color (int);
 static chtype first_char (int);
 static chtype second_char (map_val);
-static void draw_loc (const int);
 static void draw_map (void);
 static void draw_statusline (void);
-static enum refresh_amount move_curs (int, int);
-static enum refresh_amount resolve_update (Action);
+static void move_curs (int, int);
 static enum refresh_amount read_char (void);
-static void game_loop (void);
 
 static bool use_color = FALSE;
 static WINDOW *map_pad, *status_line;
@@ -63,9 +65,13 @@ static WINDOW *map_pad, *status_line;
 static int map_x, map_y;
 static int map_max_x, map_max_y;
 static int curs_x, curs_y;
+static int map_physical_right, map_physical_bottom;
+static int player_mask;
+static int ask_quit;
 
 extern void
-curses_game_init ()
+curses_game_init (info)
+     union conn_info *info;
 {
   const chtype turn_text[] = {'t', 'u', 'r', 'n', '\0'};
   const chtype cross_text[] = {'c', 'r', 'o', 's', 's', 'e', 's', '\0'};
@@ -81,16 +87,24 @@ curses_game_init ()
   if (use_color = has_colors ())
     {
       start_color ();
-      /* Hard-coded color values here.  */
+      /* Hard-coded color values here.  Having these user-configurable would
+	 mean yet another .*rc file, which I decided not to have.  If you
+	 dislike my choices, then recompile.  Besides I'm too lazy to do any
+	 rc stuff.  */
       init_pair (COL_SEA, COLOR_BLUE, COLOR_BLACK);
       init_pair (COL_ME, COLOR_YELLOW, COLOR_BLACK);
+      init_pair (COL_REMOTE, COLOR_GREEN, COLOR_BLACK);
       init_pair (COL_OTHER, COLOR_WHITE, COLOR_BLACK);
+#if 0
       /* These two are yet unimplemented.  */
       init_pair (COL_ENC, COLOR_RED, COLOR_BLACK);
       init_pair (COL_O_ENC, COLOR_MAGENTA, COLOR_BLACK);
+#endif
       init_pair (COL_CAP, COLOR_CYAN, COLOR_BLUE);
+      init_pair (COL_R_CAP, COLOR_GREEN, COLOR_BLUE);
       init_pair (COL_O_CAP, COLOR_WHITE, COLOR_BLUE);
       init_pair (COL_OCC, COLOR_CYAN, COLOR_MAGENTA);
+      init_pair (COL_R_OCC, COLOR_GREEN, COLOR_RED);
       init_pair (COL_O_OCC, COLOR_WHITE, COLOR_RED);
     }
   map_pad = newpad (game_map.height, game_map.width*2);
@@ -99,13 +113,32 @@ curses_game_init ()
   mvwaddchnstr (status_line, 0, (COLS - 8), turn_text, 4);
   mvwaddchnstr (status_line, 0, (COLS - 24), cross_text, 7);
 
-  game_loop ();
+  /* No need to worry about these having negative values.  */
+  map_max_x = game_map.width - (COLS/2-1);
+  map_max_y = game_map.height - (LINES-1);
+  map_physical_right = COLS/2-1 < game_map.width ? COLS :
+    (game_map.width+game_map.width);
+  map_physical_bottom = LINES-1 < game_map.height ? LINES :
+    (game_map.height+game_map.height);
+
+  /* Bogus refresh.  I guess I'm mutilating curses somehow here...  */
+  refresh ();
+
+  player_mask = info->common.conn_mask;
+  info->common.fd = STDIN_FILENO;
+
+  do_refresh (ALL);
 }
 
-static void
+extern void
 kill_curses ()
 {
-  endwin ();
+  static curses_killed = 0;
+  if (!curses_killed)
+    {
+      curses_killed = 1;
+      endwin ();
+    }
 }
 
 static chtype
@@ -116,12 +149,24 @@ choose_color (loc)
     return COLOR_PAIR (COL_SEA);
   if ((game_map.map[loc] & MASK_PLAYER) == game_map.turn)
     {
-      if (game_map.map[loc] & MASK_CAPITAL)
-	return COLOR_PAIR (COL_CAP);
-      else if (game_map.map[loc] & MASK_OCCUPIED)
-	return COLOR_PAIR (COL_OCC);
+      if (1<<(game_map.turn-1) & player_mask)
+	{
+	  if (game_map.map[loc] & MASK_CAPITAL)
+	    return COLOR_PAIR (COL_CAP);
+	  else if (game_map.map[loc] & MASK_OCCUPIED)
+	    return COLOR_PAIR (COL_OCC);
+	  else
+	    return COLOR_PAIR (COL_ME);
+	}
       else
-	return COLOR_PAIR (COL_ME);
+	{
+	  if (game_map.map[loc] & MASK_CAPITAL)
+	    return COLOR_PAIR (COL_R_CAP);
+	  else if (game_map.map[loc] & MASK_OCCUPIED)
+	    return COLOR_PAIR (COL_R_OCC);
+	  else
+	    return COLOR_PAIR (COL_REMOTE);
+	}
     }
   else
     {
@@ -163,8 +208,8 @@ second_char (val)
   return 'a'-1+pl - (occ ? 32 : 0);
 }
 
-static void
-draw_loc (i)
+extern void
+c_draw_loc (i)
      const int i;
 {
   chtype color = 0;
@@ -185,7 +230,7 @@ draw_map ()
 
   wmove (map_pad, 0, 0);
   for (i = 0 ; i < game_map.size; i++)
-    draw_loc (i);
+    c_draw_loc (i);
 }
 
 static void
@@ -238,13 +283,12 @@ draw_statusline ()
   mvwaddch (status_line, 0, COLS-16, ('0'+game_map.n_cross) | A_BOLD);
   if (update & UPDATE_TURN)
     mvwaddch (status_line, 0, COLS-1, (second_char (game_map.turn |
-						    MASK_OCCUPIED) |
-				       A_BOLD));
+						    MASK_OCCUPIED) | A_BOLD));
 }
 
 /* TODO: a routine to scroll wrapped maps.  Low priority, to do when I
    feel like it.  */
-static enum refresh_amount
+static void
 move_curs (x, y)
      int x, y;
 {
@@ -292,124 +336,167 @@ move_curs (x, y)
 	    }
 	}
     }
-  return refresh;
+  do_refresh (refresh);
 }
 
-static enum refresh_amount
-resolve_update (act)
-     Action act;
+extern void
+c_process_action (act)
+     Action *act;
 {
-  if (act.type == 0)
-    return NONE;
-  if (!(act.type & (EVENT_ENCIRCLEMENT|EVENT_NEWTURN)))
+  if (act->type == 0)
+    return;
+  if (!(act->type & (EVENT_ENCIRCLEMENT|EVENT_NEWTURN)))
     {
-      if (act.count == 0)
-	return STATUSLINE;
-      while (act.count-- > 0)
+      if (act->count == 0)
 	{
-	  wmove (map_pad, act.loc[act.count].y,
-		 act.loc[act.count].x+act.loc[act.count].x);
-	  draw_loc (parse_real_coord (act.loc[act.count]));
+	  do_refresh (STATUSLINE);
+	  return;
 	}
-      return REFRESH_MAP;
+      while (act->count-- > 0)
+	{
+	  wmove (map_pad, act->loc[act->count].y,
+		 act->loc[act->count].x+act->loc[act->count].x);
+	  c_draw_loc (parse_real_coord (act->loc[act->count]));
+	}
+      do_refresh (REFRESH_MAP);
+      return;
     }
-  return ALL;
+  do_refresh (ALL);
 }
 
-/* TODO: User-defineable keys.  */
-static enum refresh_amount
-read_char ()
+/* TODO: User-defineable keys.  In the meantime just edit the source and
+   compile.  */
+extern void
+c_read_key (info)
+     union conn_info *info;
 {
   int key;
 
-  while (1)
+  while ((key = wgetch (map_pad)) == ERR);
+  if (key == 12)	/* C-l pressed.  */
+    do_refresh (REFRESH_MAP);
+  if (ask_quit)
     {
-      while ((key = wgetch (map_pad)) == ERR);
-      switch (key)
+      if (key == 'y')
 	{
-	  Action act;
-	  Coord cross_loc;
-	case KEY_A1:
-	  return move_curs (-1, -1);
-	case KEY_UP:
-	  return move_curs (0, -1);
-	case KEY_A3:
-	  return move_curs (1, -1);
-	case KEY_RIGHT:
-	  return move_curs (1, 0);
-	case KEY_C3:
-	  return move_curs (1, 1);
-	case KEY_DOWN:
-	  return move_curs (0, 1);
-	case KEY_C1:
-	  return move_curs (-1, 1);
-	case KEY_LEFT:
-	  return move_curs (-1, 0);
-	case KEY_B2:
-	case ' ':
-	  cross_loc.x = curs_x % game_map.width;
-	  cross_loc.y = curs_y % game_map.height;
-	  if (toggle_cross (cross_loc))
-	    {
-	      Coord curs_coord;
-	      curs_coord.x = curs_x;
-	      curs_coord.y = curs_y;
-	      wmove (map_pad, curs_y, curs_x+curs_x);
-	      draw_loc (parse_coord (curs_coord));
-	      return REFRESH_MAP;
-	    }
-	  else
-	    return NONE;
-	case 'z':
-	case KEY_F(1):
-	  action (MODE_ATT, &act);
-	  return resolve_update (act);
-	case 'x':
-	case KEY_F(2):
-	  action (MODE_DEF, &act);
-	  return resolve_update (act);
-	case 'c':
-	case KEY_F(3):
-	  action (MODE_GUE, &act);
-	  return resolve_update (act);
-	case 12:	/* C-l pressed.  */
-	  return ALL;
-	}
-    }
-}
-
-static void
-game_loop ()
-{
-  enum refresh_amount refresh = ALL;
-  int map_physical_right, map_physical_bottom;
-
-  /* No need to worry about these having negative values.  */
-  map_max_x = game_map.width - (COLS/2-1);
-  map_max_y = game_map.height - (LINES-1);
-  map_physical_right = COLS/2-1 < game_map.width ? COLS :
-    (game_map.width+game_map.width);
-  map_physical_bottom = LINES-1 < game_map.height ? LINES :
-    (game_map.height+game_map.height);
-
-  refresh ();			/* Bogus refresh.  */
-  while (1)
-    {
-      switch (refresh)
-	{
-	case ALL:
-	  draw_map ();
-	case REFRESH_MAP:
-	  pnoutrefresh (map_pad, map_y, map_x*2, 1, 0,
-			map_physical_bottom, map_physical_right);
-	case STATUSLINE:
-	  draw_statusline ();
-	  wnoutrefresh (status_line);
-	case CURSOR:
-	  move (curs_y-map_y+1, (curs_x-map_x)*2);
+	  pid_t pid;
+	  connection_terminated (info);
+#ifndef _POSIX_JOB_CONTROL
+	  mvaddstr (1, 0, "Players still playing, please don't interrupt");
+	  mvaddstr (2, 0, "(I thought everyone had POSIX job control by now...)");
 	  refresh ();
-	  doupdate ();
+#endif
+	  /* If we are here, then some X guys are still around.  */
+	  kill_curses ();
+#ifdef _POSIX_JOB_CONTROL
+	  pid = fork ();	/* Yuck, but I want to go to background.  */
+	  /* Probably ok to die, things are bad if the fork fails.  */
+	  if (pid == -1)
+	    die ("failed to quit gracefully");
+	  else if (pid > 0)	/* Parent dies.  */
+	    _exit (0);
+	  /* Child goes on.  */
+	  setsid ();	/* Disown the console.  */
+#endif
 	}
-      refresh = read_char ();
+      else
+	{
+	  ask_quit = 0;
+	  do_refresh (REFRESH_MAP);
+	}
+      return;
+    }
+  else if (key == 'q')
+    {
+      ask_quit = 1;
+      do_refresh (CURSOR);
+      return;
+    }
+
+  if (!(player_mask & 1<<(game_map.turn-1)))
+    return;
+  switch (key)
+    {
+      Action act;
+      Coord cross_loc;
+    case KEY_A1:
+      move_curs (-1, -1);
+      break;
+    case 'j':
+    case KEY_UP:
+      move_curs (0, -1);
+      break;
+    case KEY_A3:
+      move_curs (1, -1);
+      break;
+    case 'l':
+    case KEY_RIGHT:
+      move_curs (1, 0);
+      break;
+    case KEY_C3:
+      move_curs (1, 1);
+      break;
+    case 'k':
+    case KEY_DOWN:
+      move_curs (0, 1);
+      break;
+    case KEY_C1:
+      move_curs (-1, 1);
+      break;
+    case 'h':
+    case KEY_LEFT:
+      move_curs (-1, 0);
+      break;
+    case KEY_B2:
+    case ' ':
+      cross_loc.x = curs_x % game_map.width;
+      cross_loc.y = curs_y % game_map.height;
+      if (toggle_cross (cross_loc))
+	{
+	  Coord dim = {0, 0};
+	  wmove (map_pad, curs_y, curs_x+curs_x);
+	  signal_mapchange (cross_loc, cross_loc);
+	  signal_turnwinchange ();
+	}
+      break;
+    case 'z':
+    case KEY_F(1):
+      action (MODE_ATT, &act);
+      signal_action (&act);
+      break;
+    case 'x':
+    case KEY_F(2):
+      action (MODE_DEF, &act);
+      signal_action (&act);
+      break;
+    case 'c':
+    case KEY_F(3):
+      action (MODE_GUE, &act);
+      signal_action (&act);
+      break;
+    }
+}
+
+extern void
+do_refresh (refresh)
+     const enum refresh_amount refresh;
+{
+  /* Fall-throughs intentional, don't panic.  */
+  switch (refresh)
+    {
+    case ALL:
+      draw_map ();
+    case REFRESH_MAP:
+      pnoutrefresh (map_pad, map_y, map_x*2, 1, 0,
+		    map_physical_bottom, map_physical_right);
+    case STATUSLINE:
+      draw_statusline ();
+      wnoutrefresh (status_line);
+    case CURSOR:
+      if (ask_quit)
+	mvaddstr (1, 0, "Really quit? (y/n)");
+      move (curs_y-map_y+1, (curs_x-map_x)*2);
+      refresh ();
+      doupdate ();
     }
 }
