@@ -27,7 +27,7 @@ enum fill_op {
   BREAK
 };
 
-extern enum fill_op fill (Coord, enum fill_op (Coord),
-			  enum fill_op (Coord));
+extern void init_fill ();
+extern enum fill_op fill (Coord, enum fill_op (Coord), enum fill_op (Coord));
 
 #endif /* fill.h */
