@@ -24,40 +24,30 @@
 #include <dirent.h>
 
 #include "wrappers.h"
-#include "thread.h"
+#include "comm.h"
 #include "map.h"
-#ifdef HAVE_LIBCURSES
-#include "curses_if.h"
-#endif
+#include "occupy.h"
+#include "fill.h"
 
+static enum conn_type get_type (const char);
 static int parse_numbers (const char **, const int);
-static void parse_players (int, char **, thread_func ***, const char ***);
+static void parse_players (int, char **, enum conn_type **, const char ***,
+			   int **, const int);
 static void help_msg (void);
 static void version_msg (void);
-static void get_type (const char, thread_func **);
 static const char **display_name = NULL;
 static const char *display_name_local;
 static enum player_type *connection_types;
 
-#if defined HAVE_LIBCURSES && defined HAVE_LIBX11
-static int use_curses;
-#endif
-
-#ifdef HAVE_LIBPTHREAD
-static void
-get_type (type, func)
+static enum conn_type
+get_type (type)
      const char type;
-     thread_func **func;
 {
 #ifdef HAVE_LIBX11
   if (type == ':')
-    {
-      *func = x_game_init;
-      n_x_connect++;
-    }
-  else
-    die ("invalid player type");
+    return CONN_X;
 #endif
+  die ("invalid player type");
 }
 
 static int
@@ -97,23 +87,23 @@ parse_numbers (string, this_connect)
 }
 
 static void
-parse_players (argc, argv, type, name)
+parse_players (argc, argv, type, name, connect_mask, use_curses)
      int argc;
      char *argv[];
-     thread_func ***type;
+     enum conn_type **type;
      const char ***name;
+     int **connect_mask;
+     const int use_curses;
 {
   int players_marked = 0, local_player, this_connect = 0;
   const char *ptr;
   *name = my_malloc (sizeof (char *) * argc);
-  *type = my_malloc (sizeof (thread_func *) * argc);
+  *type = my_malloc (sizeof (enum conn_type) * argc);
   n_connect = 0;
   /* Enough in all cases.  */
-  connect_mask = my_malloc (sizeof (int) * (argc+1));
-  global_conn_info = my_malloc (sizeof (union conn_info *) *
-				(argc+1));
-  global_conn_type = my_malloc (sizeof (enum conn_type) * (argc+1));
+  *connect_mask = my_malloc (sizeof (int) * (argc+1));
 
+#if HAVE_LIBX11
   while (optind < argc)
     {
       int this_display_mask;
@@ -126,44 +116,42 @@ parse_players (argc, argv, type, name)
 
       players_marked |= this_display_mask;
       (*name)[this_connect] = ptr;	/* Damn C!  :-) */
-      get_type (*(ptr-1), &((*type)[this_connect++]));
-      connect_mask[n_connect++] = this_display_mask;
+      (*type)[this_connect++] = get_type (*(ptr-1));
+      (*connect_mask)[n_connect++] = this_display_mask;
     }
+#endif
 
   /* All players, which aren't explicitly set, are local players.  */
   local_player = ~players_marked & ((1<<game_map.players)-1);
-  if (local_player)
+  /* Allow curses even if there are no players playing on console.  */
+  if (use_curses)
+    (*type)[n_connect] = CONN_CURSES;
+  else if (local_player)
     {
-#ifdef HAVE_LIBX11
-      (*type)[n_connect] = x_game_init;
+      (*type)[n_connect] = CONN_X;
       (*name)[n_connect] = "";
-#endif
-      connect_mask[n_connect++] = local_player;
     }
+  else
+    return;
+  (*connect_mask)[n_connect++] = local_player;
 }
-#endif	/* HAVE_LIBPTHREAD */
 
 static void
 help_msg ()
 {
-#ifdef HAVE_LIBPTHREAD
+#if defined HAVE_LIBX11
   puts ("Usage: motti map [options] [player, ...]");
-#ifdef HAVE_LIBCURSES
-  puts ("       motti map --curses\n");
 #else
-  puts ("");
-#endif
-#else
-  puts ("Usage: motti map [options]\n");
+  puts ("Usage: motti map [options]");
 #endif
 #if defined HAVE_LIBCURSES && defined HAVE_LIBX11
   puts ("  -c, --curses                 use curses interface");
 #endif
   puts ("      --help                   display this help and exit");
   puts ("      --version                output version information and exit\n");
-#ifdef HAVE_LIBPTHREAD
+#ifdef HAVE_LIBX11
   puts ("Player specified with format 1,2,...,n:display");
-#endif /* have_libx11 */
+#endif	/* have_libx11 */
   puts ("Report bugs to bug-motti@gnu.org");
   exit (0);
 }
@@ -203,8 +191,9 @@ main (argc, argv)
      char *argv[];
 {
   char *status_msg;
-  int use_x = 0;
-  thread_func **connection_type;
+  int use_curses = 0;
+  int *connect_mask;
+  enum conn_type *connection_type;
   const char **connection_name;
 
   while (1)
@@ -241,7 +230,7 @@ main (argc, argv)
     }
 
   puts (VERSION_STR);
-  puts ("Copyright 1999 Free Software Foundation, Inc.");
+  puts ("Copyright 2000 Free Software Foundation, Inc.");
   puts ("This is free software with ABSOLUTELY NO WARRANTY.");
   puts ("For details read file 'COPYING'.");
 
@@ -249,27 +238,17 @@ main (argc, argv)
     list_maps ();
   load_map (argv[optind++]);
 
-#ifdef HAVE_LIBPTHREAD
-#ifdef HAVE_LIBCURSES
-  if (!use_curses)
-#endif
-    parse_players (argc, argv, &connection_type, &connection_name);
-#endif /* HAVE_LIBPTHREAD */
+  parse_players (argc, argv, &connection_type, &connection_name,
+		 &connect_mask, use_curses);
 #ifdef HAVE_LIBX11
   global_argc = argc;
   global_argv = argv;
-#endif /* HAVE_LIBX11 */
+#endif
 
   srand (time ((time_t *) NULL));
 
-#ifdef HAVE_LIBX11
-#ifdef HAVE_LIBCURSES
-  if (!use_curses)
-#endif
-    create_threads (connection_type, connection_name);
-#endif
-#if HAVE_LIBCURSES
-  curses_game_init ();
-#endif
+  init_occupy ();
+  init_fill ();
+  init_game (connection_type, connection_name, connect_mask);
   return 0;	/* Never executed.  */
 }
