@@ -100,7 +100,7 @@ enum cross_act {
   NO_CROSS, IS_CROSS, ANY_CROSS, ONLY_GUERILLA
 };
 
-extern char * load_map (char *);
+extern void load_map (char *);
 extern void get_map_real_coord (Coord *);
 extern int parse_coord (Coord);
 extern int parse_real_coord (Coord);
