@@ -33,7 +33,6 @@ enum connect_type {
 #include "fill.h"
 #include "occupy.h"
 #include "wrappers.h"
-#include "thread.h"
 
 static enum fill_op borders (Coord);
 static enum fill_op mark_connected (Coord);
@@ -41,14 +40,13 @@ static enum fill_op blocked (Coord);
 static enum fill_op mark_passed (Coord);
 static enum fill_op not_connected (Coord);
 static enum fill_op mark_border (Coord);
-static int search_encircled (void);
+static int search_encircled ();
 static int fill_check (void);
 static int deadlock (Coord, int, int *, unsigned char *,
 		     void (*) (Coord, int, int *));
 static void count_connects (Coord, int, int *);
 static void comp_inv_dist (Coord, int, int *);
 static void invade (Coord);
-static void destroy (void);
 static void occupy (Action *, int *);
 static enum fill_op search_contact (Coord);
 static void loop_by_bit_mask (char *, int);
@@ -58,6 +56,64 @@ static int new_turn (void);
 static enum connect_type *connect = NULL;
 static int player;
 
+extern void
+init_occupy ()
+{
+  connect = my_malloc (sizeof (enum connect_type) * game_map.size);
+}
+
+extern void
+defeat_players (pl_mask, act)
+     const int pl_mask;
+     Action *act;
+{
+  register int i, j;
+  int have_defeats = 0, need_turnchange = 0;
+
+  for (i = 1; i <= game_map.players; i++)
+    if (1<<(i-1) & pl_mask)
+      {
+	int mask = ~(1<<(i-1));
+	if (!have_defeats)
+	  {
+	    for (j = 0; j < game_map.size; j++)
+	      connect[j] = CONNECTED;
+	    have_defeats = 1;
+	  }
+	if (game_map.turn == i)
+	  need_turnchange = 1;
+
+	game_map.active_players &= mask;
+	game_map.remaining_players &= mask;
+	game_map.n_active_players--;
+	game_map.players_left--;
+	for (j = 0; j < game_map.size; j++)
+	  if (game_map.map[j] == SEA_VAL
+	      || (game_map.map[j] & MASK_PLAYER) == i)
+	    connect[j] = UNCONNECTED;
+      }
+
+  act->type = 0;
+  act->count = 0;
+  if (have_defeats)
+    {
+      if (game_map.remaining_players > 1)
+	{
+	  player = 0;
+	  while (search_encircled ());
+	  if (need_turnchange)
+	    {
+	      if (new_turn ())
+		act->type = EVENT_NEWTURN|EVENT_UNOCCUPY|EVENT_DEFEAT;
+	      else
+		act->type = EVENT_NEWTURN|EVENT_DEFEAT;
+	      return;
+	    }
+	}
+      act->type = EVENT_DEFEAT;
+    }
+}
+
 /* Mark all other players' occupied cells, which are next to any cells passed
    on the first pass.  This leaves islands of other players' areas inside the
    encircled player's encircled area unaccounted.  */
@@ -163,11 +219,13 @@ mark_border (loc)
 	{
 	  enum connect_type this_loc_connect;
 	  this_loc_connect = connect[parse_coord (this_coord)];
-	  if (this_loc_connect == CONNECTED)	/* Neighbouring connected
-						   cell.  */
+	  /* Neighbouring connected cell.  */
+	  if (this_loc_connect == CONNECTED)
 	    {
 	      connect[loc_ref] = BORDER;
-	      if ((game_map.map[loc_ref] & MASK_PLAYER) == player)
+	      if ((game_map.map[loc_ref] & MASK_PLAYER) == player
+		  /* For general defeats.  */
+		  || (player == 0 && game_map.map[loc_ref] != SEA_VAL))
 		invade (loc);	/* Encircled player's cell.  */
 	      return UNSUCCESSFUL;
 	    }
@@ -188,11 +246,12 @@ search_encircled ()
   for (i = 0; i < game_map.size; i++)
     {
       if (connect[i] == UNCONNECTED
-	  && (game_map.map[i] & MASK_PLAYER) == player)
+	  && ((game_map.map[i] & MASK_PLAYER) == player
+	      || (player == 0 && game_map.map[i] != SEA_VAL)))
 	{
 	  /* Run only if necessary, since this is a quite expensive function.
 	     First pass was in function fill_check.  */
-	  if (!ran_2nd_pass)
+	  if (!ran_2nd_pass && player != 0)
 	    {
 	      register int j;
 	      fill (game_map.capital[player-1], &borders, &mark_connected);
@@ -293,8 +352,9 @@ count_connects (loc, pattern, array)
 	    }
 	}
     }
-  for (i = 0; i < game_map.players; i++)
-    array[i] *= i == game_map.turn-1 ? 3 : 2;
+  if (player != 0)
+    for (i = 0; i < game_map.players; i++)
+      array[i] *= i == game_map.turn ? 3 : 2;
 }
 
 static void
@@ -349,8 +409,8 @@ destroy ()
   register int i;
   game_map.players_left--;
   /* Disable the defeated player.  */
-  game_map.active_players ^= 1<<(player - 1);
-  game_map.remaining_players ^= 1<<(player - 1);
+  game_map.active_players &= ~(1<<(player - 1));
+  game_map.remaining_players &= ~(1<<(player - 1));
   game_map.n_active_players--;
   /* Mark all cells of the defeated player and seas to be unconnected.  */
   for (i = 0; i < game_map.size; i++)
@@ -370,7 +430,7 @@ occupy (act, cross_n)
     {
       map_val *map_ptr;
 
-      game_map.bit_cross ^= 1<<*cross_n;
+      game_map.bit_cross &= ~(1<<*cross_n);
       game_map.n_cross--;
       map_ptr = &game_map.map[parse_real_coord
 			     (game_map.cross[*cross_n])];
@@ -400,8 +460,8 @@ occupy (act, cross_n)
 	    connect[j] = UNCONNECTED;
 	}
       player = i;
-      /* TODO: a quick check to avoid the expensive fill_check (),
-	 when its absolutely certain it's not needed.  */
+      /* FIXME: a quick check to avoid the expensive fill_check (),
+	 when it is absolutely certain it's not needed.  */
       if (fill_check ())
 	act->type |= EVENT_ENCIRCLEMENT;
     }
@@ -420,10 +480,6 @@ action (action, act)
       act->type = 0;
       return;
     }
-  connect = my_malloc (sizeof (enum connect_type) * game_map.size);
-#ifdef HAVE_LIBPTHREAD
-  get_write_lock ();
-#endif
   switch (action)
     {
     case MODE_ATT:
@@ -526,11 +582,6 @@ action (action, act)
 
   if ((act->type & EVENT_NEWTURN) && new_turn ())
     act->type |= EVENT_UNOCCUPY;
-#ifdef HAVE_LIBPTHREAD
-  release_write_lock ();
-#endif
-  free (connect);
-  connect = NULL;
 }
 
 static enum fill_op
@@ -601,7 +652,7 @@ disable_player ()
     {
       /* Reset all occupied cells to unoccupied status.  Change
 	 turn to the next player in turn to begin.  */
-      game_map.n_active_players = game_map.players;
+      game_map.n_active_players = game_map.players_left;
       game_map.active_players = game_map.remaining_players;
       loop_by_bit_mask (&game_map.begin_turn,
 			game_map.remaining_players);
@@ -609,7 +660,7 @@ disable_player ()
       clear_map_bit ((map_val) ~(MASK_OCCUPIED | MASK_CROSS));
       return 1;
     }
-  game_map.active_players ^= 1<<(game_map.turn - 1);
+  game_map.active_players &= ~(1<<(game_map.turn - 1));
 
   return 0;
 }
