@@ -23,6 +23,7 @@
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/Xresource.h>
+#include <X11/Xatom.h>
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -41,8 +42,14 @@
 #include "map.h"
 #include "xinit.h"
 #include "wrappers.h"
-#include "thread.h"
+#include "comm.h"
 
+int global_argc = 0;
+char **global_argv = 0;
+
+static void create_DB (struct x_info_struct *);
+static int alloc_colors (struct x_info_struct *);
+static void open_windows (struct x_info_struct *);
 static enum mouse_action parse_mouse_string (const char *);
 static void init_DB_tab (struct x_info_struct *);
 static void * x_handler (void *);
@@ -57,20 +64,37 @@ static int alloc_colors (struct x_info_struct *);
 static void make_map_stipples (struct x_info_struct *,Pixmap, Pixmap, int,
 			       int);
 static void make_att_buttons (struct x_info_struct *);
-#ifdef HAVE_LIBPTHREAD
 static void make_dis_att_buttons (struct x_info_struct *);
-#endif
 static void make_att_pixs (struct x_info_struct *);
 static void make_def_pixs (struct x_info_struct *);
 static void make_gue_pixs (struct x_info_struct *);
 static void calculate_extents (struct x_info_struct *, const Font);
 static void alloc_gcs (struct x_info_struct *);
-#if 0
-static void open_windows (struct x_info_struct *);
-#endif
 static void set_hints (struct x_info_struct *, const int, const int,
 		       const int);
 
+extern int
+x_game_init (disp_name, x_info)
+     const char *disp_name;
+     struct x_info_struct *x_info;
+{
+  x_info->display = XOpenDisplay (disp_name);
+  if (!x_info->display)
+    {
+      if (n_connect == 1 && !*disp_name)
+	return 0;
+      else
+	die ("can't open display %s", disp_name);
+    }
+  x_info->fd = ConnectionNumber (x_info->display);
+
+  create_DB (x_info);
+  if (!alloc_colors (x_info))
+    die ("error allocating colors");
+  open_windows (x_info);
+  return 1;
+}
+
 static enum mouse_action
 parse_mouse_string (act_name)
      const char *act_name;
@@ -200,7 +224,7 @@ init_DB_tab (x_info)
     }
 }
 
-extern void
+static void
 create_DB (x_info)
      struct x_info_struct *x_info;
 {
@@ -297,7 +321,7 @@ alloc_Xrm_color (x_info, instance, class, deflt, color)
   return 1;
 }
 
-extern int
+static int
 alloc_colors (x_info)
      struct x_info_struct *x_info;
 {
@@ -452,7 +476,6 @@ make_att_buttons (x_info)
     }
 }
 
-#ifdef HAVE_LIBPTHREAD
 static void
 make_dis_att_buttons (x_info)
      struct x_info_struct *x_info;
@@ -465,7 +488,6 @@ make_dis_att_buttons (x_info)
 	       x_info->db_tab.but_height, 0, 0);
   XSetFunction (x_info->display, x_info->tempgc, GXcopy);
 }
-#endif
 
 static void
 make_att_pixs (x_info)
@@ -502,32 +524,33 @@ make_att_pixs (x_info)
 					x_info->db_tab.but_width,
 					x_info->db_tab.but_height,
 					x_info->depth);
-#ifdef HAVE_LIBPTHREAD
-  x_info->d_att_pix[1] = XCreatePixmap (x_info->display, x_info->main_win,
-					x_info->db_tab.but_width,
-					x_info->db_tab.but_height,
-					x_info->depth);
-  x_info->d_att_pix[2] = XCreatePixmap (x_info->display, x_info->main_win,
-					x_info->db_tab.but_width,
-					x_info->db_tab.but_height,
-					x_info->depth);
-  x_info->d_att_pix[3] = XCreatePixmap (x_info->display, x_info->main_win,
-					x_info->db_tab.but_width,
-					x_info->db_tab.but_height,
-					x_info->depth);
-  x_info->d_att_pix[4] = XCreatePixmap (x_info->display, x_info->main_win,
-					x_info->db_tab.but_width,
-					x_info->db_tab.but_height,
-					x_info->depth);
-  x_info->d_att_pix[5] = XCreatePixmap (x_info->display, x_info->main_win,
-					x_info->db_tab.but_width,
-					x_info->db_tab.but_height,
-					x_info->depth);
-  x_info->d_att_pix[6] = XCreatePixmap (x_info->display, x_info->main_win,
-					x_info->db_tab.but_width,
-					x_info->db_tab.but_height,
-					x_info->depth);
-#endif	/* HAVE_LIBPTHREAD */
+  if (n_connect > 1)
+    {
+      x_info->d_att_pix[1] = XCreatePixmap (x_info->display, x_info->main_win,
+					    x_info->db_tab.but_width,
+					    x_info->db_tab.but_height,
+					    x_info->depth);
+      x_info->d_att_pix[2] = XCreatePixmap (x_info->display, x_info->main_win,
+					    x_info->db_tab.but_width,
+					    x_info->db_tab.but_height,
+					    x_info->depth);
+      x_info->d_att_pix[3] = XCreatePixmap (x_info->display, x_info->main_win,
+					    x_info->db_tab.but_width,
+					    x_info->db_tab.but_height,
+					    x_info->depth);
+      x_info->d_att_pix[4] = XCreatePixmap (x_info->display, x_info->main_win,
+					    x_info->db_tab.but_width,
+					    x_info->db_tab.but_height,
+					    x_info->depth);
+      x_info->d_att_pix[5] = XCreatePixmap (x_info->display, x_info->main_win,
+					    x_info->db_tab.but_width,
+					    x_info->db_tab.but_height,
+					    x_info->depth);
+      x_info->d_att_pix[6] = XCreatePixmap (x_info->display, x_info->main_win,
+					    x_info->db_tab.but_width,
+					    x_info->db_tab.but_height,
+					    x_info->depth);
+    }
 
   /* The depth of the gc has to be 1.  */
   XFreeGC (x_info->display, x_info->tempgc);
@@ -571,10 +594,9 @@ make_att_pixs (x_info)
 
   /* Draw attack buttons with number of remaining attacks.  */
   make_att_buttons (x_info);
-#ifdef HAVE_LIBPTHREAD
   /* Make inverse attack buttons with information about remaining attacks.  */
-  make_dis_att_buttons (x_info);
-#endif
+  if (n_connect > 1)
+    make_dis_att_buttons (x_info);
 }
 
 static void
@@ -750,6 +772,7 @@ set_hints (x_info, min_width, min_height, geom_specified)
      struct x_info_struct *x_info;
      const int min_width, min_height, geom_specified;
 {
+  Atom save_atom;
   XWMHints *wm_hints;
   XClassHint *class_hints;
   XTextProperty window_name_prop, icon_name_prop;
@@ -795,13 +818,17 @@ set_hints (x_info, min_width, min_height, geom_specified)
   XFree (class_hints);
   XFree (window_name_prop.value);
   XFree (icon_name_prop.value);
+
+  save_atom = XInternAtom (x_info->display, "WM_DELETE_WINDOW", False);
+  XSetWMProtocols (x_info->display, x_info->main_win, &save_atom, 1);
 }
 
-extern void
+static void
 open_windows (x_info)
      struct x_info_struct *x_info;
 {
   XSetWindowAttributes win_attribs;
+  register int i;
   int x = 0, y = 0, width, height, geom_specified, y_below_buttons;
 
   /* There must be room for action buttons above the map.  */
@@ -886,6 +913,27 @@ open_windows (x_info)
 		   &win_attribs);
 
   set_hints (x_info, width, height, geom_specified);
+
+  x_info->last_sel = MODE_ATT;
+  x_info->last_active = -1;
+  XSelectInput (x_info->display, x_info->main_win, 0);
+  XSelectInput (x_info->display, x_info->turn_win, ExposureMask);
+  XSelectInput (x_info->display, x_info->map_win, ExposureMask);
+
+  x_info->uses_def_mode = 0;
+  for (i = 0; i < 5; i++)
+    if (x_info->db_tab.mouse[i] == act)
+      {
+	x_info->uses_def_mode = 1;
+	break;
+      }
+
+  XMapWindow (x_info->display, x_info->map_win);
+  XMapWindow (x_info->display, x_info->turn_win);
+  XMapWindow (x_info->display, x_info->but_win[ATT]);
+  XMapWindow (x_info->display, x_info->but_win[DEF]);
+  XMapWindow (x_info->display, x_info->but_win[GUE]);
+  XMapWindow (x_info->display, x_info->main_win);
 }
 
 #if 0
